# imgv editing options code by Ryan Kulla, rkulla@gmail.com
import gl
from wallpaper import windows_wallpaper, unix_wallpaper
from input_box import ask
from rm_img import command_delete_img
from show_message import show_message
from confirm import get_confirmation
from img_screen import get_center, my_update_screen
from usr_event import check_quit, hit_key, left_click
from cursor import normal_cursor, wait_cursor, hover_cursor#
from sys import platform
import os
import pygame.font, pygame.event, pygame.mouse
from pygame.locals import K_ESCAPE, MOUSEBUTTONDOWN, MOUSEMOTION, KEYDOWN, K_UP, K_DOWN, K_RIGHT, K_LEFT, K_RETURN, K_SPACE, K_1, K_2, K_3, K_4, K_KP1, K_KP2, K_KP3, K_KP4


FIRST_RECT1 = SECOND_RECT1 = THIRD_RECT1 = FOURTH_RECT1 = (-1, -1, -1, -1)


def command_edit_menu(screen, file, new_img, num_imgs, rect):
    menu_items = []
    screen.fill(gl.BLACK)
    pygame.display.flip()
    font = pygame.font.Font(gl.FONT_NAME, 10)
    font.set_underline(1)
    font.set_bold(1)
    esc = font.render("CLOSE", 1, gl.CANCEL_BUTTONCOLOR)
    esc_rect = esc.get_rect()
    esc_rect[0] = 20
    screen.blit(esc, esc_rect)
    pygame.display.update(esc_rect)
    show_message(screen, "%s" % os.path.basename(gl.files[file]), "top", 12, "bold")
    pygame.event.set_blocked(MOUSEMOTION)
    while 1:
       event = pygame.event.poll()
       pygame.time.wait(1)
       cursor = pygame.mouse.get_pos()
       hover_fx(screen, menu_items, cursor, font)
       check_quit(event)
 
       if hit_key(event, K_ESCAPE):
           update_edit_screen(screen, file, new_img)
           return (new_img, new_img, new_img, file, num_imgs, rect)
       if left_click(event):
           if esc_rect.collidepoint(cursor):
               update_edit_screen(screen, file, new_img)
               return (new_img, new_img, new_img, file, num_imgs, rect)

       if event.type == KEYDOWN and event.key in (K_1, K_2, K_3, K_4, K_KP1, K_KP2, K_KP3, K_KP4):
           if hit_key(event, K_1) or hit_key(event, K_KP1):
               (new_img, new_img, new_img, file, num_imgs, rect) = do_delete_image(screen, new_img, file, rect, num_imgs)
           if hit_key(event, K_2) or hit_key(event, K_KP2): 
               do_set_wallpaper(screen, file, new_img, num_imgs, rect)
           if hit_key(event, K_3) or hit_key(event, K_KP3):
                do_external_viewer(screen, file, new_img)
           if hit_key(event, K_4) or hit_key(event, K_KP4):
               (new_gamma) = do_gamma(screen, file, new_img)
               if new_gamma != None:
                   try:
                       pygame.display.set_gamma(float(new_gamma))
                   except:
                       print 'invalid gamma value: %s' % new_gamma
                   update_edit_screen(screen, file, new_img)
           return (new_img, new_img, new_img, file, num_imgs, rect)
                   
       if event.type == MOUSEBUTTONDOWN and pygame.mouse.get_pressed()[0]:
           for it in menu_items:
               if it[0].collidepoint(cursor) and it[1] in men_ops:
                   if it[1] == "1) Delete image":
                       (new_img, new_img, new_img, file, num_imgs, rect) = do_delete_image(screen, new_img, file, rect, num_imgs)
                   if it[1] == "2) Set as wallpaper": 
                       do_set_wallpaper(screen, file, new_img, num_imgs, rect)
                   if it[1] == "3) Open in external viewer":
                        do_external_viewer(screen, file, new_img)
                   if it[1] == "4) Adjust gamma":
                       (new_gamma) = do_gamma(screen, file, new_img)
                       if new_gamma != None:
                           try:
                               pygame.display.set_gamma(float(new_gamma))
                           except:
                               print 'invalid gamma value: %s' % new_gamma
                           update_edit_screen(screen, file, new_img)
                   return (new_img, new_img, new_img, file, num_imgs, rect)
       (menu_items, men_ops) = edit_menu(screen, file, menu_items)


def do_delete_image(screen, new_img, file, rect, num_imgs):
    fn = gl.files[file]
    answer = get_confirmation(screen, "Delete %s? [y/n]" % os.path.basename(fn))
    if answer == "yes":
        (new_img, new_img, new_img, file, num_imgs, rect) = command_delete_img(fn, new_img,\
        screen, file, rect)
        update_edit_screen(screen, file, new_img)
        return (new_img, new_img, new_img, file, num_imgs, rect)
    if answer == "no":
        update_edit_screen(screen, file, new_img)
        return (new_img, new_img, new_img, file, num_imgs, rect)
    return (new_img, new_img, new_img, file, num_imgs, rect)


def do_set_wallpaper(screen, file, new_img, num_imgs, rect):
    wait_cursor()
    if platform == 'win32':
        windows_wallpaper(file)
    else:
        unix_wallpaper(file)
    update_edit_screen(screen, file, new_img)
    

def do_external_viewer(screen, file, new_img):
    wait_cursor()
    cmd = "%s %s" % (gl.EXTERNAL_VIEWER, gl.files[file])
    try:
        os.system(cmd)
    except:
        pass
    update_edit_screen(screen, file, new_img)
    

def do_gamma(screen, file, new_img):
    screen.fill(gl.BLACK)
    pygame.display.flip()
    show_message(screen, "Enter a new gamma value. Example: 1.0 (do not use the numeric keypad)", 20, 15, ("bold"))
    new_gamma = ask(screen, "New gamma value ")
    return (new_gamma)

                       
def edit_menu(screen, file, menu_items):
    font = pygame.font.Font(gl.FONT_NAME, 18)
    font.set_bold(1)
    men_ops = ["1) Delete image", "2) Set as wallpaper", "3) Open in external viewer", "4) Adjust gamma"]
    line = 65
    col = 20
    normal_cursor()
    for m in men_ops:
        ren_name = m
        ren = font.render(ren_name, 1, gl.BLUE, gl.FONT_BG)
        ren_rect = ren.get_rect()
        ren_rect[0] = col
        ren_rect[1] = line
        menu_items.append((ren_rect, m))
        screen.blit(ren, ren_rect)
        line = line + 30
        pygame.display.update(ren_rect)
    return (menu_items, men_ops)


def update_edit_screen(screen, file, new_img):
    rect = get_center(screen, new_img)
    num_imgs = len(gl.files)
    my_update_screen(new_img, screen, rect, file, num_imgs)
    normal_cursor()


def hover_fx(screen, menu_items, cursor, font):
    global FIRST_RECT1, SECOND_RECT1, THIRD_RECT1, FOURTH_RECT1
    for it in menu_items:
        if it[0].collidepoint(cursor):
            if it[1] == "1) Delete image":
                blank_fx(screen, 1)
                FIRST_RECT1 = (it[0][0] - 10, it[0][1] + (font.size(it[1])[1] / 2), it[0][2], it[0][3])
                show_message(screen, "+", FIRST_RECT1, 12, ("bold"))
                show_message(screen, "               Delete image off disk               ", "bottom", 12)
                break
            elif it[1] == "2) Set as wallpaper":
                blank_fx(screen, 2)
                SECOND_RECT1 = (it[0][0] - 10, it[0][1] + (font.size(it[1])[1] / 2), it[0][2], it[0][3])
                show_message(screen, "+", SECOND_RECT1, 12, ("bold"))
                show_message(screen, "          Set image as desktop wallpaper          ", "bottom", 12)
                break
            elif it[1] == "3) Open in external viewer":
                blank_fx(screen, 3)
                THIRD_RECT1 = (it[0][0] - 10, it[0][1] + (font.size(it[1])[1] / 2), it[0][2], it[0][3])
                show_message(screen, "+", THIRD_RECT1, 12, ("bold"))
                viewer = os.path.basename(gl.EXTERNAL_VIEWER)
                viewer = viewer[:viewer.rfind('.')]
                if gl.EXTERNAL_VIEWER in ('"None"', '"none"'):
                    show_message(screen, "                    No viewer specified                     ", "bottom", 12)
                else:
                    show_message(screen, "                    Open in %s                    " % viewer, "bottom", 12)
                break
            elif it[1] == "4) Adjust gamma":
                blank_fx(screen, 4)
                FOURTH_RECT1 = (it[0][0] - 10, it[0][1] + (font.size(it[1])[1] / 2), it[0][2], it[0][3])
                show_message(screen, "+", FOURTH_RECT1, 12, ("bold"))
                show_message(screen, "               Gamma correction               ", "bottom", 12)
                break


def blank_fx(screen, row):
    if row != 1:
        show_message(screen, "  ", FIRST_RECT1, 12, ("bold"))
    if row != 2:
        show_message(screen, "  ", SECOND_RECT1, 12, ("bold"))
    if row != 3:
        show_message(screen, "  ", THIRD_RECT1, 12, ("bold"))
    if row != 4:
        show_message(screen, "  ", FOURTH_RECT1, 12, ("bold"))
