# imgv data objects code by Ryan Kulla, rkulla@gmail.com
import os
from sys import argv
from cfg import get_conf_name, read_config_file
from dir_nav import strip_dirs, get_imgs
from error_box import errorbox
from pygame import version


BASE_DIR = os.getcwd()
try:
    DATA_DIR = os.environ['IMGV_HOME'] + os.sep + 'data' + os.sep
except KeyError:
    DATA_DIR = os.path.join(BASE_DIR, 'data' + os.sep)


TRANS_FX = "NONE"
LAST_DIR = ""
ADDED_DIR_NUMS = 0
BEEN_THERE_DONE_THAT = 0
ALREADY_DOWNLOADED = 0
ESCAPED = 0
MY_KEYDOWN = 0
MY_KEYUP = 0
MY_KEYRIGHT = 0
MY_KEYLEFT = 0
FONT_NAME = DATA_DIR + "Vera.ttf"
IMGV_COLOR = 0, 0, 0
WHITE = 255, 255, 255
MID_GRAY = 192, 192, 192
SILVER = 200, 200, 200
BLACK = 0, 0, 0
BLUE = 0, 0, 255
RED = 255, 000, 051
PURPLE = 102, 000, 255
SKY_BLUE = 000, 255, 255
LIGHT_GREEN = 0, 255, 102
YELLOW = 255, 255, 51
ORANGE = 255, 102, 051
GREEN = 0, 136, 0
NAVY = 0, 0, 18
DARK_SLATE_BLUE = 72, 41, 139
DARK_SLATE_GRAY = 49, 79, 79
SADDLE_BROWN = 139, 69, 19
COLORS = {"RED":RED, "BLUE":BLUE, "PURPLE":PURPLE, "SKY_BLUE":SKY_BLUE, "SILVER":SILVER,\
"GREEN":GREEN, "LIGHT_GREEN":LIGHT_GREEN, "ORANGE":ORANGE, "YELLOW":YELLOW, "BLACK":BLACK, "WHITE":WHITE,\
"NAVY":NAVY, "DARK_SLATE_BLUE":DARK_SLATE_BLUE, "DARK_SLATE_GRAY":DARK_SLATE_GRAY, "SADDLE_BROWN":SADDLE_BROWN, "MID_GRAY":MID_GRAY}
FONT_BG = BLACK
FONT_BG_NEED_CHANGE = 0
BUTTON_BGCOLOR = LIGHT_GREEN
BUTTON_HOVERCOLOR = SKY_BLUE
BUTTON_TEXTCOLOR = BLACK
CANCEL_BUTTONCOLOR = BLUE
MAX_SCREEN_FILES = 0 # number of image names file_master can show on a given screen res 
MAX_SF = {"640x480":165, "800x600":301, "1024x768":513, "1280x1024":858}
CONF_FILE = get_conf_name()
WRAP = "1"
WRAP_SLIDESHOW = "1"
THUMB_VAL = "100x100"
THUMBING = 0
USING_SCROLL_MENU = 0 # sentinel for TOGGLE_TRANSPARENT
MOVIES_VAL = "1"
KEEP_MENU_OPEN = "1"
COUNT_CLICKS = 0
FIT_IMAGE_VAL = "1"
FIT_IMAGE_SLIDESHOW_VAL = "1"
SCALE_UP = 0
START_DIRECTORY_VAL = os.getcwd()
START_FULLSCREEN = 1
CORRECT_PASSWORD = None
read_config_file()
MOVIE_FILE = DATA_DIR + "movie-file.jpg"
SLIDE_SHOW_RUNNING = 0
TRANSPARENT = 1, 1, 1
TOGGLE_TRANSPARENT = 1
TOGGLE_STATUS_BAR = 1
MENU_COLOR_ITEM = 0
MENU_ADJUST = COLORS.values()
MENU_ADJUST.remove(IMGV_COLOR) # or wont be able to see it
MENU_ADJUST.remove(NAVY)
NS_GLOBAL = 0 # number of seconds it took to load. used with check_timer()
START_RES = IMGV_RESOLUTION
MENU_POS = START_RES[0] - 150
IMGV_LOGO = DATA_DIR + "imgv-logo1.jpg"
NO_MATCHES_IMG = DATA_DIR + "no-matches.jpg"
TITLE = "imgv"
IMGV_PLAYLISTS = DATA_DIR + "playlists"
PLAY_LIST_NAME = " "
REMOTE = 0 # true if its a remote url to the image
REMOTE_IMG = "" # remote image name
REMOTE_IMG_DATA = "" # remote image data
REMOTE_FILE_SIZE = 0 # remote image byte size
MIN_WIDTH = 10
MIN_HEIGHT =10 
MAX_WIDTH = 11760
MAX_HEIGHT = 2000
MOVE = 21 # how much to move bigger-than-window images by
ERROR_IMG = DATA_DIR + "imgv-error.jpg" 
PYTHON_POWERED_IMG = DATA_DIR + "PythonPowered.gif"#
DRIVE = os.getcwd()[0] # used for multi-drive support
REAL_WIDTH = 0
REAL_HEIGHT = 0
CUR_PATH = ""
SHRUNK = 0
SKIP_FIT = 0
THUMBING = 0
MULTI_VIEWING = 0
SKIP = 0 # used for long strings in inputbox()
REFRESH_IMG_COUNT = 1
BEING_HOVERED = 0
ISURL = 0
URL = ""
SHOW_DATA = False
INDEX_OF = False
JUST_SOURCE = 0 # for complete url image links
URL_ERROR = False
MULT_DIRS = [] # to store multiple tagged directories
FILTER_COMMAND = {}
SUBDIRS = 0
SORT_HIT = 0
SHOW_ESC_MSG = 1


IMG_TYPES = [".gif",".GIF", ".jpg",".JPG", ".jpeg",".JPEG", ".png",".PNG", ".bmp",".BMP", ".ppm",".PPM", ".pcx",".PCX", 
".tga",".TGA", ".tif",".TIF", ".tiff",".TIFF", ".pnm",".PNM", ".pbm",".PBM", ".pgm",".PGM", ".xpm",".XPM", ".xcf",".XCF",
".lbm",".LBM", ".iff",".IFF"]


if MOVIES_VAL == 1:
    for t in (".mpg", ".MPG", ".mpeg", ".MPEG"):
        IMG_TYPES.append(t)


MENU_ITEMS = [
    " Image Details ",
    "",
    " Next Image ",
    " Previous Image ",
    " First Image ",
    " Last Image ",
    " Shuffle ",
    " Unshuffle ",
    "",
    " Directory Browser ",
    " Image Browser ",
    " Extract From Web ",
    "",
    " Thumbnails ",
    " Four At A Time ",
    " Slide Show ",
    " Playlist Options ",
    " Add To Playlist ",
    "", 
    " Close Image ",
    " Hide Image ",
    " Flip Horizontal ",
    " Flip Vertical ",
    " Rotate CW ",
    " Rotate CCW ",
    " Zoom In ",
    " Zoom Out ",
    "",
    " Resize Window ",
    " Font Color ",
    " Transparent ",
    " Status Bars ",
    " Refresh ",
    "",
    " Close Menu ",
    " Edit ",
    " Help ",
    " Exit "]


if not version.ver >= "1.1":
    errorbox("Version Error", "You need pygame 1.1 or greater")


if len(argv) < 2:
    if START_DIRECTORY_VAL == "CURRENT":
        dir_or_file = '.'
    else:
        dir_or_file = START_DIRECTORY_VAL
else:
    dir_or_file = argv[1]


# set initial directory values
CACHE_DIRS = os.listdir(START_DIRECTORY_VAL)
CACHE_DIRS = strip_dirs(CACHE_DIRS)
CACHE_DIRS.sort()
CACHE_DIR_OK = 0


if os.path.isdir(dir_or_file) or os.path.isdir(dir_or_file + os.sep):
    os.chdir(dir_or_file)
    DRIVE = dir_or_file.split(":")[0]
    # store only image files in 'files'
    files = get_imgs(os.getcwd(), 0)
elif os.path.isfile(dir_or_file):
    if dir_or_file[1] == ":" and dir_or_file[2] == os.sep:
        os.chdir(os.path.dirname(dir_or_file))
    files = [dir_or_file]
elif dir_or_file[:5] == "http:":
    REMOTE = 1
    files = [dir_or_file]
else:
    errorbox("Invalid File or Directory", dir_or_file)


files.sort() # put images in alphabetical order

