# help screen code by Ryan Kulla, rkulla@gmail.com
import gl
from usr_event import check_quit
from cursor import normal_cursor, hover_cursor
from about import command_about
from img_screen import get_center, my_update_screen
from show_message import show_message
import pygame.font, pygame.event
from pygame.display import flip, update
from pygame.locals import KEYDOWN, MOUSEBUTTONDOWN, MOUSEMOTION, K_LALT, K_RALT, K_LCTRL, K_RCTRL, K_TAB, K_ESCAPE, K_SPACE
from random import shuffle
from usr_event import left_click, hit_key
import webbrowser


def command_help(screen, new_img, file, rect, num_imgs):
    command_about(screen, new_img, rect, file, num_imgs)
    help(screen)
    rect = get_center(screen, new_img)
    my_update_screen(new_img, screen, rect, file, len(gl.files))


def help(screen):
    screen.fill(gl.BLACK)
    pygame.display.update()    
    show_message(screen, "Main Keyboard Commands", "top", 11, ("italic", "bold", "underline"))
    key_list = [" Space, N, Ctrl+Tab = Next image, Backspace, B = Previous Image ", 
                " D = Change Directory ", 
                " I = Image Browser ", 
                " T = Thumbnails (Use Space/Right-click and Backspace/Middle-click to navigate. Space to pause)", 
                " 4 = View 4 images at a time (Use Space/Right-click and Backspace/Middle-click to navigate. W to slideshow)", 
                " W = Slide Show ", 
                " P = Add to Playlist, Ctrl+P = Playlist Options ", 
                " C = Close Menu",
                " F = First Image (Jump to the first image), L = Last Image (Jump to the last image)", 
                " + = Zoom In, - = Zoom Out ", 
                " R = Rotate CW, Ctrl+R = Rotate CCW ",
                " Esc = Refresh (Resets current image to original state or Reloads after directory changes)",
                " M = Flip Horizontal (mirror), V = Flip Vertical ", 
                " S = Shuffle, U = Unshuffle ", 
                " A = Download Image (Saves remote images to your imgv download directory)", 
                " Del, Ctrl+W = Close Image, Ctrl+Del = Permanently deletes from harddisk", 
                " X = Hide Image, Ctrl+X = Hide Status Bars ", 
                " O = Open URL (Extract image(s) from a website) ",
                " F1 = 640x480, F2 = 800x600, F3 = 1024x768, F4 = 1280x1024, F5 = Toggle Fullscreen, F6 = Resize Win ", 
                " H = Help (This message) ", 
                " Q = Exit (Works in most screens)", 
                " Arrow keys = Pan Image (Left, Right, Up and Down) ", 
                " E = Edit ", 
                " Z = Image Details ",
                " 1 = Toggle scaling images to fit the window for the current session. ",
                " Ctrl+0 (Ctrl+Zero) = Fit image to the window, Alt+0 = Actual Pixels (show image at its real size) "]
 
    key_list.sort()
    font = pygame.font.Font(gl.FONT_NAME, 12)
    close_font = pygame.font.Font(gl.FONT_NAME, 12)
    plots = get_plots(key_list)
    start_pos = 345
    
    for pos, line in enumerate(key_list):
        show_message(screen, line, plots[pos], 11)
 
    close_font.set_underline(1)
    close_font.set_bold(1)
    esc = close_font.render("CLOSE", 1, gl.CANCEL_BUTTONCOLOR)
    esc_rect = esc.get_rect()
    esc_rect[0] = 20
    screen.blit(esc, esc_rect)
    pygame.display.update(esc_rect)
        
    gl.SHOW_MESSAGE_COLOR = gl.BLUE
    doc_msg = "Click here to view the online documentation - http://imgv.sf.net/doc/"#
    doc_rect = show_message(screen, doc_msg, ((screen.get_width() / 2) - (font.size(doc_msg)[0] / 2), 400, 0, 0), 12, ("underline", "bold"))#    
    more_info_msg = "Click here to view my software blog - http://ryanspicks.blogspot.com/"
    blog_rect = show_message(screen, more_info_msg, ((screen.get_width() / 2) - (font.size(more_info_msg)[0] / 2), 450, 0, 0), 12, ("underline", "bold"))

    normal_cursor()
    pygame.event.set_blocked(MOUSEMOTION)
    while 1:
        event = pygame.event.poll()
        pygame.time.wait(1)
        cursor = pygame.mouse.get_pos()
        check_quit(event)
        if blog_rect.collidepoint(cursor) or doc_rect.collidepoint(cursor):
            hover_cursor()
        else:
            normal_cursor()
        if hit_key(event, K_ESCAPE) or hit_key(event, K_SPACE):
           gl.SHOW_MESSAGE_COLOR = gl.MENU_COLOR
           break
        if left_click(event):
            if blog_rect.collidepoint(cursor):
                webbrowser.open("http://ryanspicks.blogspot.com/", 1, 1)
            elif doc_rect.collidepoint(cursor):
                webbrowser.open("http://imgv.sourceforge.net/doc/", 1, 1)#
            elif esc_rect.collidepoint(cursor):
                gl.SHOW_MESSAGE_COLOR = gl.MENU_COLOR
                break


def get_plots(the_list):
    "return a list of random plot positions to write to"
    total_items = len(the_list)
    spacer = 13
    j = 0
    plots = []
    for i in range(spacer * total_items):
        j = j + spacer
        plots.append(j)
        if j == spacer * total_items:
            break
    shuffle(plots)
    return plots
