#! /usr/bin/env python


""" imgv.py 
    Description: Image viewing application
    Version: 3.1
    Author: Ryan Kulla 
    Site: http://imgv.sourceforge.net/
    Email: rkulla@gmail.com """


from sys import platform
from os import environ
# use windib when the default is directx:
if platform == 'win32': 
    environ['SDL_VIDEODRIVER'] = 'windib'
import pygame.event, pygame.time
from pygame.display import set_caption, set_mode
from pygame.locals import  MOUSEMOTION, MOUSEBUTTONDOWN, Rect, KEYDOWN, KEYUP
import gl
from load_timers import start_timer, check_timer
from img_screen import init_screen, get_center, my_update_screen
from show_message import show_message
from cursor import wait_cursor, normal_cursor
from usr_event import check_quit, right_click
from load_img import load_img
from handle_keyboard import handle_keyboard
from main_menu import command_main_menu
from pan import command_down, command_up, command_right, command_left
from res import command_fullscreen

def main():
    pygame.time.delay(5) # to make start_timer() work initially
    start = start_timer()
    num_imgs = len(gl.files) # total number of images
    pygame.init() # seems needed for mac osx
    init_screen()
    wait_cursor()
    screen = pygame.display.set_mode(gl.START_RES)
    set_caption(gl.TITLE)
    if gl.REMOTE == 1:
        show_message(screen, "Loading image. Please wait..", 34, 42)
    file = 0
    if num_imgs < 1:
        gl.files = [gl.IMGV_LOGO]
        num_imgs = 1 
        img = load_img(gl.files[file], screen)
    else:
        img = load_img(gl.files[file], screen)
    wait_cursor()
    img_width = img.get_width()
    img_height = img.get_height()
    refresh_img = img
    new_img = img
    rect = get_center(screen, new_img)
    ns = check_timer(start)
    my_update_screen(new_img, screen, rect, file, num_imgs, ns)
    normal_cursor()
    if gl.START_FULLSCREEN == 1:
        command_fullscreen(screen, new_img, file, num_imgs, rect)
    # start with menu open
    screen_midtop = screen.get_rect().midtop
    screen_width = screen.get_width()
    screen_height = screen.get_height()
    new_img_width = new_img.get_width()
    new_img_height = new_img.get_height()
    (refresh_img, screen, screen_midtop, file, num_imgs, new_img, img,\
    new_img_width, new_img_height, rect) = command_main_menu(refresh_img, screen,\
    screen_midtop, file, len(gl.files), rect, new_img, img, new_img_width, new_img_height)

    pygame.event.set_blocked(MOUSEMOTION)
    # main loop
    while 1:
        events = pygame.event.get()
        pygame.time.wait(1)# use this so pygame doesn't use 100% CPU
        last_rect = Rect(rect)
        screen_midtop = screen.get_rect().midtop
        screen_width = screen.get_width()
        screen_height = screen.get_height()
        new_img_width = new_img.get_width()
        new_img_height = new_img.get_height()
    
        for event in events:
            if event.type == KEYDOWN:
                (screen, rect, new_img, img, refresh_img, file, num_imgs, screen_midtop,\
                screen_width, screen_height, new_img_width, new_img_height, last_rect) =\
                handle_keyboard(event, screen, rect, new_img, img, refresh_img, file, len(gl.files),\
                screen_midtop, screen_width, screen_height, new_img_width, new_img_height, last_rect)
            if event.type == KEYUP:
                 stop_auto_repeat()
            check_quit(event)
            if event.type == MOUSEBUTTONDOWN:
                if right_click(event):
                    (refresh_img, screen, screen_midtop, file, num_imgs, new_img, img,\
                    new_img_width, new_img_height, rect) = command_main_menu(refresh_img, screen,\
                    screen_midtop, file, num_imgs, rect, new_img, img, new_img_width, new_img_height)
        if gl.KEEP_MENU_OPEN == "1" and gl.COUNT_CLICKS == 1: # Re-open the purposely closed window that frees up RAM
            gl.COUNT_CLICKS = 0
            (refresh_img, screen, screen_midtop, file, num_imgs, new_img, img,\
            new_img_width, new_img_height, rect) = command_main_menu(refresh_img, screen,\
            screen_midtop, file, num_imgs, rect, new_img, img, new_img_width, new_img_height)
        start_auto_repeat(rect, last_rect, new_img, screen, file, len(gl.files), screen_midtop, screen_width, screen_height)
    clean_screen()


def start_auto_repeat(rect, last_rect, new_img, screen, file, num_imgs, screen_midtop, screen_width, screen_height):
    if gl.MY_KEYDOWN == 1:
        command_down(rect, last_rect, new_img, screen, file, num_imgs, screen_midtop)
    if gl.MY_KEYUP == 1:
        command_up(rect, last_rect, new_img, screen, file, num_imgs, screen_midtop, screen_height)
    if gl.MY_KEYRIGHT == 1:
        command_right(rect, last_rect, new_img, screen, file, num_imgs, screen_midtop)
    if gl.MY_KEYLEFT == 1:
        command_left(rect, last_rect, new_img, screen, file, num_imgs, screen_midtop, screen_width)   


def stop_auto_repeat():
    gl.MY_KEYDOWN, gl.MY_KEYUP, gl.MY_KEYRIGHT, gl.MY_KEYLEFT = 0, 0, 0, 0


if __name__=='__main__': 
    main()
