# imgv window resolution resizing code by Ryan Kulla, rkulla@gmail.com
import gl
from cursor import wait_cursor, normal_cursor
from show_message import show_message
from img_screen import my_update_screen, get_center
from cursor import normal_cursor
from input_box import ask
from usr_event import check_quit, hit_key, left_click
import pygame.mouse, pygame.event, pygame.font
from pygame.display import set_mode, toggle_fullscreen
from pygame.locals import FULLSCREEN, MOUSEBUTTONDOWN, MOUSEMOTION, K_ESCAPE, KEYDOWN, K_F1, K_F2, K_F3, K_F4, K_F5, K_c


FIRST_RECT1 = SECOND_RECT1 = THIRD_RECT1 = FOURTH_RECT1 = FIFTH_RECT1 = SIXTH_RECT1 = (-1, -1, -1, -1)


def resize_it(event, screen, new_img, file, num_imgs):
    resolution = event.size
    screen = set_mode(resolution)
    rect = get_center(screen, new_img)
    screen_midtop = screen.get_rect().midtop
    my_update_screen(new_img, screen, rect, file, num_imgs)
    return (screen_midtop, rect)


def command_640x480(new_img, file, num_imgs, rect):
    "switch to 640x480"
    gl.MAX_SCREEN_FILES = gl.MAX_SF["640x480"]
    wait_cursor()
    resolution = 640, 480
    gl.IMGV_RESOLUTION = resolution
    screen = set_mode(resolution)
    rect = get_center(screen, new_img)
    screen_midtop = screen.get_rect().midtop
    my_update_screen(new_img, screen, rect, file, num_imgs)
    normal_cursor()
    return (screen_midtop, rect)


def command_800x600(new_img, file, num_imgs, rect):
    "switch to 800x600"
    gl.MAX_SCREEN_FILES = gl.MAX_SF["800x600"]
    wait_cursor()
    resolution = 800, 600
    gl.IMGV_RESOLUTION = resolution
    screen = set_mode(resolution)
    rect = get_center(screen, new_img)
    screen_midtop = screen.get_rect().midtop
    my_update_screen(new_img, screen, rect, file, num_imgs)
    normal_cursor()
    return (screen_midtop, rect)


def command_1024x768(new_img, file, num_imgs, rect):
    "switch to 1024x768"
    gl.MAX_SCREEN_FILES = gl.MAX_SF["1024x768"]
    wait_cursor()
    resolution = 1024, 768
    gl.IMGV_RESOLUTION = resolution
    screen = set_mode(resolution)
    rect = get_center(screen, new_img)
    screen_midtop = screen.get_rect().midtop
    my_update_screen(new_img, screen, rect, file, num_imgs)
    normal_cursor()
    return (screen_midtop, rect)


def command_1280x1024(new_img, file, num_imgs, rect):
    "switch to 1280x1024"
    gl.MAX_SCREEN_FILES = gl.MAX_SF["1280x1024"]
    wait_cursor()
    resolution = 1280, 1024
    gl.IMGV_RESOLUTION = resolution
    screen = set_mode(resolution)
    rect = get_center(screen, new_img)
    screen_midtop = screen.get_rect().midtop
    my_update_screen(new_img, screen, rect, file, num_imgs)
    normal_cursor()
    return (screen_midtop, rect)


def command_custom_res(screen, new_img, file, num_imgs, rect, resolution):
    "switch to customxcustom"
    gl.MAX_SCREEN_FILES = gl.MAX_SF["640x480"]
    wait_cursor()
    gl.IMGV_RESOLUTION = resolution
    screen = set_mode(resolution)
    rect = get_center(screen, new_img)
    screen_midtop = screen.get_rect().midtop
    my_update_screen(new_img, screen, rect, file, num_imgs)
    normal_cursor()
    return (screen_midtop, rect)


def command_fullscreen(screen, new_img, file, num_imgs, rect):
    "toggle between full screen and last screen mode"
    wait_cursor()
    if not toggle_fullscreen():
        set_mode(screen.get_size(), screen.get_flags() ^FULLSCREEN, screen.get_bitsize())
        my_update_screen(new_img, screen, rect, file, num_imgs)
    normal_cursor()


def command_show_res_modes(screen, new_img, file, num_imgs, rect):
    screen.fill(gl.BLACK)
    pygame.display.flip()
    menu_items = []
    font = pygame.font.Font(gl.FONT_NAME, 10)
    font.set_underline(1)
    font.set_bold(1)
    res_font = pygame.font.Font(gl.FONT_NAME, 18)
    res_font.set_bold(1)
    esc = font.render("CLOSE", 1, gl.CANCEL_BUTTONCOLOR)
    esc_rect = esc.get_rect()
    esc_rect[0] = 20
    screen.blit(esc, esc_rect)
    pygame.display.update(esc_rect)
    show_message(screen, "Choose a window size/resolution for imgv", "top", 12, ("underline", "bold"))
    pygame.event.set_blocked(MOUSEMOTION)
    while 1:
       event = pygame.event.poll()
       pygame.time.wait(1)
       cursor = pygame.mouse.get_pos()
       hover_fx(screen, menu_items, cursor, res_font)
       check_quit(event)
       if hit_key(event, K_ESCAPE):
           update_res_screen(screen, file, new_img)
           return (screen.get_rect().midtop, rect)
       if left_click(event):
           if esc_rect.collidepoint(cursor):
               update_res_screen(screen, file, new_img)
               return (screen.get_rect().midtop, rect)

       if event.type == KEYDOWN and event.key in (K_F1, K_F2, K_F3, K_F4, K_F5, K_c):
           if hit_key(event, K_F1):
               (screen_midtop, rect) = command_640x480(new_img, file, num_imgs, rect)
           if hit_key(event, K_F2):
               (screen_midtop, rect) = command_800x600(new_img, file, num_imgs, rect)
           if hit_key(event, K_F3):
               (screen_midtop, rect) = command_1024x768(new_img, file, num_imgs, rect)
           if hit_key(event, K_F4):
               (screen_midtop, rect) = command_1280x1024(new_img, file, num_imgs, rect)
           if hit_key(event, K_F5):
               command_fullscreen(screen, new_img, file, num_imgs, rect)
               screen_midtop = screen.get_rect().midtop
           if hit_key(event, K_c):
               (screen_midtop, rect) = do_custom(screen, new_img, file, num_imgs, rect)
           return (screen_midtop, rect)
           
       if event.type == MOUSEBUTTONDOWN and pygame.mouse.get_pressed()[0]:
           for it in menu_items:
               if it[0].collidepoint(cursor) and it[1] in men_ops:
                   if it[1] == "F1) 640x480":
                       (screen_midtop, rect) = command_640x480(new_img, file, num_imgs, rect)
                   elif it[1] == "F2) 800x600":
                       (screen_midtop, rect) = command_800x600(new_img, file, num_imgs, rect)
                   elif it[1] == "F3) 1024x768":
                       (screen_midtop, rect) = command_1024x768(new_img, file, num_imgs, rect)
                   elif it[1] == "F4) 1280x1024":
                       (screen_midtop, rect) = command_1280x1024(new_img, file, num_imgs, rect)
                   elif it[1] == "F5) Fullscreen":
                       command_fullscreen(screen, new_img, file, num_imgs, rect)
                       screen_midtop = screen.get_rect().midtop
                   elif it[1] == "C) Custom":
                       (screen_midtop, rect) = do_custom(screen, new_img, file, num_imgs, rect)
                   return (screen_midtop, rect)
       (menu_items, men_ops) = show_res_modes(screen, menu_items, res_font)
     

def show_res_modes(screen, menu_items, font):
    men_ops = ["F1) 640x480", "F2) 800x600", "F3) 1024x768", "F4) 1280x1024", "F5) Fullscreen", "C) Custom"]
    line = 65
    col = 20
    normal_cursor()
    for m in men_ops:
        ren_name = m
        ren = font.render(ren_name, 1, gl.BLUE, gl.FONT_BG)
        ren_rect = ren.get_rect()
        ren_rect[0] = col
        ren_rect[1] = line
        menu_items.append((ren_rect, m))
        screen.blit(ren, ren_rect)
        line = line + 30
        pygame.display.update(ren_rect)
    return (menu_items, men_ops)


def hover_fx(screen, menu_items, cursor, font):
   global FIRST_RECT1, SECOND_RECT1, THIRD_RECT1, FOURTH_RECT1, FIFTH_RECT1, SIXTH_RECT1
   for it in menu_items:
       if it[0].collidepoint(cursor):
           if it[1] == "F1) 640x480":
               blank_fx(screen, 1)
               FIRST_RECT1 = (it[0][0] - 10, it[0][1] + (font.size(it[1])[1] / 2), it[0][2], it[0][3])
               show_message(screen, "+", FIRST_RECT1, 12, ("bold"))
               show_message(screen, "     Resize to 640x480         ", "bottom", 12)
               break
           elif it[1] == "F2) 800x600":
               blank_fx(screen, 2)
               SECOND_RECT1 = (it[0][0] - 10, it[0][1] + (font.size(it[1])[1] / 2), it[0][2], it[0][3])
               show_message(screen, "+", SECOND_RECT1, 12, ("bold"))
               show_message(screen, "     Resize to 800x600         ", "bottom", 12)
               break
           elif it[1] == "F3) 1024x768":
               blank_fx(screen, 3)
               THIRD_RECT1 = (it[0][0] - 10, it[0][1] + (font.size(it[1])[1] / 2), it[0][2], it[0][3])
               show_message(screen, "+", THIRD_RECT1, 12, ("bold"))
               show_message(screen, "     Resize to 1024x768        ", "bottom", 12)
               break
           elif it[1] == "F4) 1280x1024":
               blank_fx(screen, 4)
               FOURTH_RECT1 = (it[0][0] - 10, it[0][1] + (font.size(it[1])[1] / 2), it[0][2], it[0][3])
               show_message(screen, "+", FOURTH_RECT1, 12, ("bold"))
               show_message(screen, "     Resize to 1280x1024        ", "bottom", 12)
               break
           elif it[1] == "F5) Fullscreen": 
               blank_fx(screen, 5)
               FIFTH_RECT1 = (it[0][0] - 10, it[0][1] + (font.size(it[1])[1] / 2), it[0][2], it[0][3])
               show_message(screen, "+", FIFTH_RECT1, 12, ("bold"))
               show_message(screen, "    Toggle  Fullscreen        ", "bottom", 12)
               break
           elif it[1] == "C) Custom":
               blank_fx(screen, 6)
               SIXTH_RECT1 = (it[0][0] - 10, it[0][1] + (font.size(it[1])[1] / 2), it[0][2], it[0][3])
               show_message(screen, "+", SIXTH_RECT1, 12, ("bold"))
               show_message(screen, "         Custom Size          ", "bottom", 12)
               break


def blank_fx(screen, row):
    if row != 1:
        show_message(screen, "  ", FIRST_RECT1, 12, ("bold"))
    if row != 2:
        show_message(screen, "  ", SECOND_RECT1, 12, ("bold"))
    if row != 3:
        show_message(screen, "  ", THIRD_RECT1, 12, ("bold"))
    if row != 4:
        show_message(screen, "  ", FOURTH_RECT1, 12, ("bold"))
    if row != 5:
        show_message(screen, "  ", FIFTH_RECT1, 12, ("bold"))
    if row != 6:
        show_message(screen, "  ", SIXTH_RECT1, 12, ("bold"))


def update_res_screen(screen, file, new_img):
    rect = get_center(screen, new_img)
    num_imgs = len(gl.files)
    my_update_screen(new_img, screen, rect, file, num_imgs)
    normal_cursor()


def do_custom(screen, new_img, file, num_imgs, rect):
    screen.fill(gl.BLACK)
    pygame.display.flip()
    show_message(screen, "Enter a custom window size/resolution. (Example:  455x600)", "top", 12, ("bold"))
    res = ask(screen, "New size ")
    if res == None:
        return (screen.get_rect().midtop, rect)
    try:
        res = res.lower().split('x')
        res = int(res[0]), int(res[1])
        (screen_midtop, rect) = command_custom_res(screen, new_img, file, num_imgs, rect, res)
    except:
       return (screen.get_rect().midtop, rect)
    return (screen_midtop, rect)
