# image status bar code by Ryan Kulla, rkulla@gmail.com
import gl
import os
import pygame
from pygame.display import set_caption, update
from show_message import show_message
from buttons import imgv_button
import exif


def exif_data(screen, filename):
    exif_data1 = []
    exif_data2 = []
    try:
        file = open(filename, 'rb')
    except:
        return
    data = exif.process_file(file)
    if not data:
        return
    x = data.keys()
    x.sort()
    for i in x:
        if i in ('JPEGThumbnail', 'TIFFThumbnail'):
            continue
        try:
            if i == "EXIF DateTimeOriginal":
                exif_data1.append(data[i].printable.replace(':', '-', 2))
            if i == "EXIF FocalLength":
                mm = data[i].printable.split('/')
                mm = float(mm[0]) / float(mm[1])
                exif_data1.append("Focal Length: %smm" % mm)
            if i == "EXIF ExposureTime": # shutter speed
                exif_data1.append("Shutter Speed: " + data[i].printable + "sec")
            if i == "EXIF FNumber":
                aperture = data[i].printable.split('/')
                aperture = float(aperture[0]) / float(aperture[1])
                exif_data1.append("Aperture: %s" % aperture)
            if i == "EXIF ExposureProgram":
                exif_data2.append("Exposure Mode: %s" % data[i].printable)
            if i == "EXIF ExposureBiasValue":
                exif_data2.append("Exposure Bias: %s" % data[i].printable)
            if i == "EXIF ISOSpeedRatings":
                exif_data1.append("ISO: %s" % data[i].printable)
            if i == "EXIF LightSource":
                exif_data1.append("White Balance: %s" % data[i].printable)
            if i == "EXIF Flash":
                exif_data1.append("Flash: %s" % data[i].printable)
            if i == "EXIF MeteringMode":
                if data[i].printable == "5":
                    exif_data2.append("Metering Mode: Multi-segment")
                else:
                    exif_data2.append("Metering Mode: %s" % data[i].printable)
            if i == "Image Make":
                exif_data2.append("Make: %s" % data[i].printable)
            if i == "Image Model":
                exif_data2.append("Model: %s" % data[i].printable)
        except:
            pass
    try:
        exif_data1 = [x + " | " for x in exif_data1[:-1]] + [exif_data1[-1]] # add a divider
        exif_data2 = [x + " | " for x in exif_data2[:-1]] + [exif_data2[-1]] 
        show_message(screen, " " + ' '.join(exif_data1) + " ", (1, 0), 9)
        show_message(screen, " " + ' '.join(exif_data2) + " ", (1, 10), 9)
    except:
        pass


def img_info(screen, screen_midtop, filename, file, num_imgs, new_img, ns):
    if gl.FIT_IMAGE_VAL == "1":
        set_caption("%s - %s     [Fit-image enabled] " % (gl.TITLE, filename))
    else:
        set_caption("%s - %s" % (gl.TITLE, filename))
    if gl.TOGGLE_STATUS_BAR:
        "display info about the current image"
        full_filename = filename
        current_img = str(file + 1)
        (img_width, img_height) = new_img.get_size()
        if gl.REMOTE:
            fsize = gl.REMOTE_FILE_SIZE
            filename = gl.REMOTE_IMG
            filename = os.path.basename(filename)
        else:
            fsize = os.path.getsize(filename)
            filename = os.path.basename(filename)
        
        if fsize <= 1024:
            file_size = "%d bytes" % fsize
        elif fsize >= 1024 and fsize <= (1024 * 1024):
            file_size = "%.1f KB" % (fsize / 1024.0)
        elif fsize >= 1024 and fsize <= (1024 * 1024 * 1024):
            file_size = "%.1f MB" % (fsize / (1024.0 * 1024.0))
        else:
            file_size = "0 bytes"     
    
        if gl.SLIDE_SHOW_RUNNING == 1:
            slide_show_running = "[slideshow]"
        else:
            slide_show_running = " "    
            
        if gl.PLAY_LIST_NAME != " ":
            img_status = " %s/%s, %s (%sx%s) - Loaded in %s secs, %s [%s] %s" % (current_img,\
            str(num_imgs), filename, img_width, img_height, str(ns), str(file_size), gl.PLAY_LIST_NAME,\
            slide_show_running)
        else:
            img_status = " %s/%s, %s (%sx%s) - Loaded in %s secs, %s %s" % (current_img,\
            str(num_imgs), filename, img_width, img_height, str(ns), str(file_size), slide_show_running)

        if not gl.TOGGLE_TRANSPARENT:
            # draw transparent 'tinted' bar to be the background for the image status message to appear on
            transren = pygame.Surface((screen.get_width(), 15)).convert_alpha()
            transren.fill([gl.BLACK[0], gl.BLACK[1], gl.BLACK[2], 90]) # RGBA (A=Alpha)
            transrect = transren.get_rect()
            transrect.midbottom = screen.get_rect().midbottom
            screen.blit(transren, transrect)
            update(transrect)

            # write the image status message
            status_font = msg_font = pygame.font.Font(gl.FONT_NAME, 10)
            ren = status_font.render(img_status, 1, gl.MENU_COLOR) 
            ren_rect = ren.get_rect()
            ren_rect.midbottom = screen.get_rect().midbottom
            screen.blit(ren, ren_rect)
            update(ren_rect)
        else:
            show_message(screen, img_status, "bottom", 10, "bold")
        exif_data(screen, full_filename)
