# image thumbnail code by Ryan Kulla, rkulla@gmail.com
import gl
from img_screen import paint_screen, get_center, my_update_screen
from usr_event import check_quit, hit_key, left_click, right_click, middle_click
from cursor import normal_cursor, wait_cursor
from load_img import load_img
import pygame.event, pygame.mouse, pygame.transform, pygame.time
from pygame.display import update, set_caption, get_caption
from pygame.locals import K_ESCAPE, K_SPACE, K_BACKSPACE, MOUSEMOTION, K_t
from os.path import basename

PAUSED = 0
OLD_CAP = ""


def command_thumbs(screen, new_img, file):
    gl.THUMBING = 1
    paint_screen(screen)
    set_caption(gl.TITLE)
    (new_img, new_img, new_img, file) = thumbs_engine(screen, new_img, file)
    rect = get_center(screen, new_img)
    my_update_screen(new_img, screen, rect, file, len(gl.files))
    normal_cursor()
    gl.THUMBING = 0
    return (new_img, new_img, new_img, file, rect)


def thumbs_engine(screen, new_img, file):
    global PAUSED#
    screen_pause = 0
    #paused = 0#
    SPACER = 10
    x = []
    place = file # start thumbing from current image position
    marker = 0
    (i, j) = (SPACER, SPACER)
    pygame.event.set_blocked(MOUSEMOTION)
    while 1:
        event = pygame.event.poll()
        pygame.time.wait(1)
        check_quit(event)
        if hit_key(event, K_ESCAPE):
            break
        if hit_key(event, K_SPACE) or hit_key(event, K_t) or left_click(event): # stop at the current thumbnail
            set_caption("[PAUSED]")
            PAUSED = 1
            screen_pause = 1
        if screen_pause == 1:
            normal_cursor()
            while 1:
                event = pygame.event.poll()
                pygame.time.wait(1)
                cursor = pygame.mouse.get_pos()
                hover_fx(screen, x, cursor)
                if left_click(event): # load clicked image
                    for item in x:
                        if item[0].collidepoint(cursor):
                            wait_cursor()
                            new_img = load_img(item[1], screen)
                            file = gl.files.index(item[1])
                            return (new_img, new_img, new_img, file)
                check_quit(event)
                if hit_key(event, K_SPACE) or hit_key(event, K_t) or right_click(event): # go to next thumb page
                    if not place >= len(gl.files):
                        if not PAUSED:
                            paint_screen(screen)
                            set_caption("[PAUSED]")
                            x = []
                            screen_pause = 0
                            break
                        set_caption("[PAUSED]")
                        PAUSED = 0
                        screen_pause = 0
                        marker = 0
                        break
                if hit_key(event, K_BACKSPACE) or middle_click(event): # go back to previous thumb page
                    if ((place - marker) > 0):
                        paint_screen(screen)
                        screen_pause = 0
                        place = place - (marker * 2)
                        marker = 0
                        x = []
                        break
                if hit_key(event, K_ESCAPE):
                    return (new_img, new_img, new_img, file)
        set_caption(gl.TITLE + " [Press SPACE to pause. ESC to cancel]  -  Loading Thumbs...") 
        (x, i, j, place, screen_pause, marker) = show_thumbs(screen, SPACER, x, i, j, place, marker)
        pygame.time.delay(5)
    return (new_img, new_img, new_img, file)


def show_thumbs(screen, SPACER, x, i, j, place, marker):
    wait_cursor()
    if place < len(gl.files):
        img_name = gl.files[place]
        load_msg = "%s (Esc to abort) Loading Thumbnails... %s: %d/%d" % (gl.TITLE, img_name, place, len(gl.files))
        img = load_img(img_name, screen)
        (w, h) = gl.THUMB_VAL.split('x')
        if img.get_width() < int(w) and img.get_height() < int(h):
            pass
        elif img.get_width() < int(w):
            img = pygame.transform.scale(img, (img.get_width(), int(h)))
        elif img.get_height() < int(h):
            img = pygame.transform.scale(img, (int(w), img.get_height()))
        else:
            img = pygame.transform.scale(img, (int(w), int(h)))
        if (i + img.get_width()) >= screen.get_width():
            i = SPACER
            j = j + (int(h) + SPACER)
        if (j + img.get_height()) >= screen.get_height():
            i, j = SPACER, SPACER
            return (x, i, j, place, 1, marker)
        img_rect = img.get_rect()
        img_rect[0] = i
        img_rect[1] = j
        x.append((img_rect, img_name))
        screen.blit(img, (i, j))
        update(img_rect)
        i = i + (int(w) + SPACER)
        place = place + 1
        marker = marker + 1
    if place >= len(gl.files):
        return (x, 0, 0, place, 1, marker)
    return (x, i, j, place, 0, marker)


def hover_fx(screen, x, cursor):
    global OLD_CAP
    for it in x:
        if it[0].collidepoint(cursor):
            if PAUSED:
                OLD_CAP = gl.TITLE + " - " + it[1] + " [PAUSED]"
            else:#
                OLD_CAP = gl.TITLE + " - " + it[1]
            if OLD_CAP != get_caption()[0]:
                set_caption(OLD_CAP)
            
