# verbose image details code by Ryan Kulla, rkulla@gmail.com
import gl
import os
from sys import platform
import imghdr
if platform != 'win32':
    from pwd import getpwuid
    from grp import getgrgid
from time import ctime, time, strptime, strftime
from stat import S_IMODE, ST_MODE, ST_MTIME, ST_ATIME, ST_CTIME, ST_UID, ST_GID
from img_screen import get_center, my_update_screen, junk_rect
from show_message import show_message
from buttons import imgv_button, hover_button
from cursor import wait_cursor, normal_cursor
from load_img import load_img
from usr_event import check_quit, hit_key
if platform == 'win32':
    import BmpImagePlugin, JpegImagePlugin, PngImagePlugin, SgiImagePlugin, SunImagePlugin, TgaImagePlugin, TiffImagePlugin, PcxImagePlugin, PpmImagePlugin, XpmImagePlugin # for py2exe to work with PIL
import Image # PIL
import pygame.font, pygame.event, pygame.transform
from pygame.display import update, flip, set_mode
from pygame.locals import KEYDOWN, K_LALT, K_RALT, K_LCTRL, K_RCTRL, K_TAB, MOUSEBUTTONDOWN, MOUSEMOTION
from types import StringType, NoneType
import exif


ROW_SEP = 19
UNIQUE_COLORS = None
SHOW_EXIFBUTTON = 1
EXIF_BUTTON_ROW = 435


class verbose:
    def __init__(self, screen, file):
        global ROW_SEP, EXIF_BUTTON_ROW
        sw, sh = screen.get_size()
        self.font_size = 14
        self.prev_pic_row = ROW_SEP + (ROW_SEP / 2) + 2
        if sw == 640 and sh == 480:
            self.font_size = 9
            ROW_SEP = 29
            EXIF_BUTTON_ROW = 435
            self.prev_pic_row = ROW_SEP + (ROW_SEP / 2) + 2
        if sw == 800 and sh == 600:
            self.font_size = 12
            ROW_SEP = 39
            EXIF_BUTTON_ROW = 454
            self.prev_pic_row = ROW_SEP + (ROW_SEP / 2) - 2
        if sw == 1024 and sh == 768:
            self.font_size = 12
            ROW_SEP = 49
            EXIF_BUTTON_ROW = 454
            self.prev_pic_row = ROW_SEP + (ROW_SEP / 2) - 3
        if sw == 1280 and sh == 1024:
            self.font_size = 12
            ROW_SEP = 59
            EXIF_BUTTON_ROW = 454
            self.prev_pic_row = ROW_SEP + (ROW_SEP / 2) - 8
        if sw > 1280 and sh > 1024:
            self.font_size = 12
            ROW_SEP = 59
            EXIF_BUTTON_ROW = 454
            self.prev_pic_row = ROW_SEP + (ROW_SEP / 2) - 8
        self.font = pygame.font.Font(gl.FONT_NAME, self.font_size)
        show_message(screen, "Image Details", "top", self.font_size, ("bold", "underline", "italic"))
        self.start_width = 250
        self.show_exif = 1
        self.row = 11
        self.screen = screen
        self.pil_info = 1
        self.bitsperpixel = None
        try:
            self.im = Image.open(gl.files[file])
        except:
            self.pil_info = 0
            self.show_exif = 0
    def filename(self, file, num_imgs):
        self.print_info("File: %s (%s/%s)" % (os.path.basename(gl.files[file]), str(file + 1), str(num_imgs)))
        self.print_info("Directory: %s" % (os.path.dirname(gl.files[file])).capitalize())
    def filesize(self, file):
        fsize = os.path.getsize(gl.files[file])
        if fsize >= 1024:
            filesize_details = "Size: %.2f KB (%d Bytes)" % ((fsize / 1024.0), fsize)
        else:
            filesize_details = "Size: %d Bytes" % fsize
        self.print_info(filesize_details)
    def img_type(self, file):
        self.img_type = imghdr.what(gl.files[file])
        if self.img_type != None:
            self.img_type = self.img_type.upper()
        if self.pil_info:
            if self.im.info.has_key("progression"):
                if type(self.img_type) != NoneType:
                    self.img_type += " (Progressive)"
        self.print_info("Type: %s" % self.img_type)
    def compression(self):
        img_format = " "
        if self.pil_info:
            if self.im.info.has_key('compression'):
                if self.img_type == "TIF" or self.img_type == "TIFF":
                    self.show_exif = 0 # compressed tiff's seem to break exif code
                compression = "Compression:  %s" % self.im.info['compression']
            else:
                compression = " "
            if self.im.info.has_key('version'): # gif
                img_format = "Format: %s.  " % self.im.info['version']
            if self.im.info.has_key('jfif'):
                compression = "Compression: JPEG"
                img_format = "Format: JFIF.  "
            if self.img_type == "PNG":
                compression = "Compression: PNG - ZIP"
            if img_format != " ":
                self.print_info(img_format + compression)
    def aspect_ratio(self):
        if self.pil_info and self.im.info.has_key('jfif_unit'):
            if self.im.info.has_key('jfif_density'):
                aspect_ratio = str(self.im.info['jfif_density'][0]) + ' x ' + str(self.im.info['jfif_density'][1])
            if self.im.info['jfif_unit'] == 0:
                self.print_info("Aspect Ratio: %s" % aspect_ratio)
            if self.im.info['jfif_unit'] == 2:
                self.print_info("Aspect Ratio: %s (Dots Per Centimeter)" % aspect_ratio)
    def bit_depth(self):
        if self.pil_info:
            if self.im.mode == "RGB" or self.im.mode == "YCbCr":
                self.bitsperpixel = 24
            elif self.im.mode == "P" or self.im.mode == "L":
                self.bitsperpixel = 8
            elif self.im.mode == "1":
                self.bitsperpixel = 1
            elif self.im.mode == "RGBA" or self.im.mode == "CMYK" or self.im.mode == "I" or self.im.mode == "F":
                self.bitsperpixel = 32
    def get_pixel_format(self):
        if self.pil_info:
            if self.bitsperpixel == 1:
                self.pixel_format = "Bilevel (Black and White)"
            elif self.bitsperpixel == 8 and self.im.mode != "P":
                self.pixel_format = "Grayscale"
            elif self.im.mode == "P":
                self.pixel_format = "Color Palette"
            else:
                self.pixel_format = self.im.mode
            self.print_info("Pixel Format: %s" % self.pixel_format)
    def colors(self):
        uniquecolors_rect = junk_rect()
        total_colors = ""
        imret = None
        if self.pil_info and self.bitsperpixel not in (None, 32):
            imret = self.im
            if self.bitsperpixel == 24:
                total_colors = "Total Colors: 16.7 Million"
            else:
                total_colors = "Total Colors: %s" % (2 ** self.bitsperpixel)
            self.print_info(total_colors)
            if UNIQUE_COLORS == None and SHOW_EXIFBUTTON and total_colors != "":
                uniquecolors_rect = imgv_button(self.screen, " Unique Colors ",\
                 (self.font.size(total_colors)[0] + 250), self.row, None)
        return (uniquecolors_rect, total_colors, self.row, self.font, imret)
    def resolution(self):
        if self.pil_info:
            if self.im.info.has_key('dpi'):
                dpi = "Resolution: %s x %s (Dots Per Inch)" % (str(self.im.info['dpi'][0]), str(self.im.info['dpi'][1]))
            else:
                dpi = " "
            self.print_info(dpi)
    def gamma(self):
        if self.pil_info:
            if self.im.info.has_key('gamma'):
                self.print_info("Gamma:  %s" % self.im.info['gamma'])
    def transparency(self):
        if self.pil_info:
            if self.im.info.has_key('transparency'):
                self.print_info("Transparency:  %s" % self.im.info['transparency'])
    def software(self):
        if self.pil_info:
            if self.im.info.has_key('Software'):
                self.print_info("Software: %s" % self.im.info['Software'])
    def dimensions(self, refresh_img):
        self.print_info("Dimensions: %s x %s x %s BPP" % (gl.REAL_WIDTH, gl.REAL_HEIGHT, self.bitsperpixel))
    def picture_taken(self, filen):
        filename = gl.files[filen]
        try:
           file = open(filename, 'rb')
        except:
            return
        data = exif.process_file(file)
        if not data:
            return
        x = data.keys()
        x.sort()
        for i in x:
            if i in ('JPEGThumbnail', 'TIFFThumbnail'):
                continue
            try:
                if i == "EXIF DateTimeOriginal":
                    date_str = strftime('%a %b %d %H:%M:%S %Y', strptime(data[i].printable, '%Y:%m:%d %H:%M:%S'))
                    self.print_info("Picture Taken: %s" % date_str)
            except:
                pass
    def last_modified(self, file):
        self.print_info("Modified: %s" % str(ctime(os.stat(gl.files[file])[ST_MTIME])))
    def last_accessed(self, file):
        self.print_info("Accessed: %s" % str(ctime(os.stat(gl.files[file])[ST_ATIME])))
    def show_current_image(self, file):
        self.current = load_img(gl.files[file], self.screen)
        (self.current, img_width, img_height) = preview_img(self.screen, self.current)
        current_rect = self.current.get_rect()
        current_rect[0] = 15
        current_rect[1] = self.prev_pic_row
        self.screen.blit(self.current, current_rect)
        update(current_rect)
        wpos = 15
        downpos = self.prev_pic_row - 2
        pygame.draw.line(self.screen, gl.SILVER, (wpos, downpos), (wpos, img_height + downpos + 2)) # left side of border
        pygame.draw.line(self.screen, gl.SILVER, (wpos + img_width, downpos), (wpos + img_width, img_height + downpos + 2)) # right side 
        pygame.draw.line(self.screen, gl.SILVER, (wpos, downpos), ((wpos + img_width), downpos)) # top of border 
        pygame.draw.line(self.screen, gl.SILVER, (wpos, img_height + downpos + 2), ((wpos + img_width), img_height + downpos + 2)) # bottom 
    def exif_data(self, filen):
        global SHOW_EXIFBUTTON
        self.screen.fill(gl.BLACK)
        pygame.display.flip()
        if not self.show_exif: 
            SHOW_EXIFBUTTON = 0
            return 
        exif_info = []
        filename = gl.files[filen]
        try:
            file = open(filename, 'rb')
        except:
            exif_info.append("%s unreadable" % filename)
            SHOW_EXIFBUTTON = 0
            return
        data = exif.process_file(file)
        if not data:
            exif_info.append("No EXIF information found")
            SHOW_EXIFBUTTON = 0
            return
        x = data.keys()
        x.sort()
        for i in x:
            if i in ('JPEGThumbnail', 'TIFFThumbnail'):
                continue
            try:
                exif_info.append('%s:  %s' % (i, data[i].printable))
            except:
                exif_info.append('error', i, '"', data[i], '"')
        SHOW_EXIFBUTTON = 0
        pos = 25
        show_message(self.screen, "Exif Information", "top", 13, ("underline", "bold"))
        try:
            for line in exif_info:
                if type(line) is StringType and len(line) <= 250: # parachutes on long lines without this
                    exif_font = pygame.font.Font(gl.FONT_NAME, 9)
                    ren = exif_font.render(line, 1, gl.MENU_COLOR)
                    ren_rect = ren.get_rect()
                    ren_rect[0] = 14
                    ren_rect[1] = pos
                    self.screen.blit(ren, ren_rect)
                    pos = pos + 9
        except:
            pass 
        flip()
    def histogram(self):
        if not self.pil_info:
            return
        w = self.screen.get_width()
        h = ROW_SEP + 415
        hist = self.im.histogram()
        redlist = hist[0:255]
        greenlist = hist[256:(256 + 255)]
        bluelist = hist[(256 + 255 + 1):]
        wdiv = 1.5 # shorten the width of the histogram
        vdiv = max(hist) / 150 # shorten the height of the histogram
        wpos = 16
        vlen = 175
        if self.pixel_format == "Grayscale":
            for i, v in enumerate(hist):
                pygame.draw.line(self.screen, gl.SILVER, ((i / wdiv) + wpos, h), ((i / wdiv) + wpos, (h - (v / vdiv))), 1)
        else:
            for i, v in enumerate(redlist):
                pygame.draw.line(self.screen, gl.RED, ((i / wdiv) + wpos, h), ((i / wdiv) + wpos, (h - (v / vdiv))), 1)
            for i, v in enumerate(greenlist):
                pygame.draw.line(self.screen, gl.GREEN, ((i / wdiv) + wpos, h), ((i / wdiv) + wpos, (h - (v / vdiv))), 1)
            for i, v in enumerate(bluelist):
                pygame.draw.line(self.screen, gl.BLUE, ((i / wdiv) + wpos, h), ((i / wdiv) + wpos, (h - (v / vdiv))), 1)
        show_message(self.screen, "Histogram", (wpos + 1, h - 174), 10, "bold")
        pygame.draw.line(self.screen, gl.SILVER, (wpos - 2, h + 2), (wpos - 2, h - vlen)) # left side of border
        pygame.draw.line(self.screen, gl.SILVER, ((i / wdiv) + wpos, h - vlen), ((i / wdiv) + wpos, h + 2)) # right side
        pygame.draw.line(self.screen, gl.SILVER, (wpos - 2, h - vlen), ((i / wdiv) + wpos, h - vlen)) # top 
        pygame.draw.line(self.screen, gl.SILVER, (wpos - 2, h + 2), ((i / wdiv) + wpos, h + 2)) # bottom 
    def print_info(self, msg):
        if msg == " ":
            return
        self.row += ROW_SEP
        ren = self.font.render(msg, 1, gl.SHOW_MESSAGE_COLOR, gl.FONT_BG)
        ren_rect = ren.get_rect()
        ren_rect[0] = self.start_width
        ren_rect[1] = self.row
        self.screen.blit(ren, ren_rect)


def command_verbose_info(screen, new_img, refresh_img, rect, file, num_imgs):
    screen.fill(gl.BLACK)
    pygame.display.flip()
    orig_resolution = screen.get_size()
    if gl.REMOTE:
        remote_img_details(screen, new_img, rect, file, num_imgs)
    else:
        verbose_info(screen, new_img, refresh_img, file, num_imgs)
    rect = get_center(screen, new_img)
    my_update_screen(new_img, screen, rect, file, num_imgs)


def remote_img_details(screen, new_img, rect, file, num_imgs):
    screen.fill(gl.BLACK)
    pygame.display.flip()
    while 1:
        event = pygame.event.wait()
        show_message(screen, gl.REMOTE_IMG, (0, 30, 0, 0), 12)
        check_quit(event)
        if event.type == KEYDOWN or event.type == MOUSEBUTTONDOWN:
            return


def preview_img(screen, refresh_img):
    (img_width, img_height) = refresh_img.get_size()
    # display image in dimensions that won't distort it:
    if img_width == img_height:    # if dimensions are equal
        if img_width < 175 and img_height < 175: # don't change if smaller
            small_img = pygame.transform.scale(refresh_img, (img_width, img_height))
        else: # shrink down
            small_img = pygame.transform.scale(refresh_img, (175, 175))
    if img_width < img_height:
        if img_width < 175 and img_height < 200: # don't change
            small_img = pygame.transform.scale(refresh_img, (img_width, img_height))
        else:
            small_img = pygame.transform.scale(refresh_img, (175, 200))
    if img_width > img_height:
        if img_width < 200 and img_height < 175: # don't change if smaller
            small_img = pygame.transform.scale(refresh_img, (img_width, img_height))
        else:
            small_img = pygame.transform.scale(refresh_img, (200, 175))
    (img_width, img_height) = small_img.get_size()
    return (small_img, img_width, img_height)


def print_verbose_info(screen, new_img, refresh_img, file, num_imgs):
    verb = verbose(screen, file)
    verb.show_current_image(file)
    verb.filename(file, num_imgs)
    verb.img_type(file)
    verb.bit_depth()
    verb.dimensions(refresh_img)
    verb.resolution()
    verb.compression()
    verb.aspect_ratio()
    verb.get_pixel_format()
    (uniquecolors_rect, total_colors, row, font, im) = verb.colors()
    verb.gamma()
    verb.transparency()
    verb.software()
    verb.filesize(file)
    verb.picture_taken(file)
    verb.last_modified(file)
    verb.last_accessed(file)
    verb.histogram()
    flip()
    return (uniquecolors_rect, total_colors, row, font, im, verb)


def verbose_info(screen, new_img, refresh_img, file, num_imgs):
    global UNIQUE_COLORS
    wait_cursor()
    screen.fill(gl.BLACK)
    pygame.display.flip()
    (uniquecolors_rect, total_colors, row, font, im, verb) = print_verbose_info(screen, new_img, refresh_img, file, num_imgs)
    if SHOW_EXIFBUTTON:
        exif_rect = imgv_button(screen, " EXIF Data ", 5, ROW_SEP + EXIF_BUTTON_ROW, None)
    return_rect = imgv_button(screen, " Return ", 5, 0, "topleft")
    normal_cursor()
    transparency = 0
    while 1:
        event = pygame.event.poll()
        pygame.time.wait(1)
        check_quit(event)
        cursor = pygame.mouse.get_pos()
        if SHOW_EXIFBUTTON:
            hover_button(exif_rect, cursor, screen, " EXIF Data ", 5, ROW_SEP + EXIF_BUTTON_ROW, None)
        if UNIQUE_COLORS == None and SHOW_EXIFBUTTON and total_colors != "":
            hover_button(uniquecolors_rect, cursor, screen, " Unique Colors ", (font.size(total_colors)[0] + 250), row, None)
        hover_button(return_rect, cursor, screen, " Return ", 5, 0, "topleft")
        if event.type == MOUSEBUTTONDOWN and pygame.mouse.get_pressed()[0]:
            if uniquecolors_rect != junk_rect():
                if uniquecolors_rect.collidepoint(cursor):
                    wait_cursor()
                    UNIQUE_COLORS = len(dict.fromkeys(im.getdata())) # determine unique colors
                    (uniquecolors_rect, total_colors, row, font, im, verb) = print_verbose_info(screen,\
                     new_img, refresh_img, file, num_imgs)
                    if not gl.TOGGLE_TRANSPARENT:
                        gl.TOGGLE_TRANSPARENT = 1
                        transparency = 1
                    show_message(screen,  "(Unique: %s)            " % UNIQUE_COLORS,\
                     ((font.size(total_colors)[0] + 260), row), 14, "italic")
                    if transparency:
                        gl.TOGGLE_TRANSPARENT = 0
                    normal_cursor()
            if exif_rect.collidepoint(cursor):
                wait_cursor()
                verb.exif_data(file)
                normal_cursor()
            if return_rect.collidepoint(cursor):
                before_exit()
                break
        if event.type == KEYDOWN and event.key not in (K_LALT, K_RALT, K_LCTRL, K_RCTRL, K_TAB):
            before_exit()
            break


def before_exit():
    global UNIQUE_COLORS, SHOW_EXIFBUTTON
    UNIQUE_COLORS = None
    SHOW_EXIFBUTTON = 1
    pygame.display.set_caption(gl.TITLE)
